const std = @import("std");

const Grid = @This();

pub const size = 16;

array: [size][size][size]Tile = .{.{.{0} ** size} ** size} ** size,

pub fn get(self: *Grid, pos: Pos) Ref {
    return .{ .grid = self, .pos = pos };
}

pub fn layerAs(self: *Grid, z: i4, comptime T: type) []T {
    const bytes = std.mem.sliceAsBytes(&self.array[@as(u4, @bitCast(z))]);
    return std.mem.bytesAsSlice(T, bytes);
}

pub const Tile = u8;

pub const Pos = packed struct {
    x: i4,
    y: i4,
    z: i4,

    pub fn init(x: i4, y: i4, z: i4) Pos {
        return .{ .x = x, .y = y, .z = z };
    }

    pub fn initTruncate(x: anytype, y: anytype, z: anytype) Pos {
        const sign = @typeInfo(@TypeOf(x, y, z)).Int.signedness;
        const T = @Type(.{ .Int = .{ .signedness = sign, .bits = 4 } });
        return .{
            .x = @bitCast(@as(T, @truncate(x))),
            .y = @bitCast(@as(T, @truncate(y))),
            .z = @bitCast(@as(T, @truncate(z))),
        };
    }
};

pub const Ref = struct {
    grid: *Grid,
    pos: Pos,

    pub fn as(self: Ref, comptime T: type) *T {
        std.debug.assert(@sizeOf(T) == @sizeOf(Tile));
        const x: usize = @as(u4, @bitCast(self.pos.x));
        const y: usize = @as(u4, @bitCast(self.pos.y));
        const z: usize = @as(u4, @bitCast(self.pos.z));
        return @ptrCast(&self.grid.array[z][y][x]);
    }
};
