const std = @import("std");
const sdl = @import("sdl");

pub const PhysicalKey = enum(u8) {
    // zig fmt: off
    backspace = 0x08,
    tab       = '\t',
    enter     = '\r',
    escape    = 0x1B,

    // Numbers
    _0 = '0', _1 = '1', _2 = '2', _3 = '3', _4 = '4',
    _5 = '5', _6 = '6', _7 = '7', _8 = '8', _9 = '9',

    // Letters
    a = 'A', b = 'B', c = 'C', d = 'D', e = 'E', f = 'F', g = 'G',
    h = 'H', i = 'I', j = 'J', k = 'K', l = 'L', m = 'M', n = 'N',
    o = 'O', p = 'P', q = 'Q', r = 'R', s = 'S', t = 'T', u = 'U',
    v = 'V', w = 'W', x = 'X', y = 'Y', z = 'Z',

    // Visual
    space         = ' ',
    grave         = '`',
    minus         = '-',
    equals        = '=',
    left_bracket  = '[',
    right_bracket = ']',
    semicolon     = ';',
    apostrophe    = '\'',
    backslash     = '\\',
    comma         = ',',
    period        = '.',
    slash         = '/',

    // Nagivation
    insert    = 0x8D,
    delete    = 0x7F,
    home      = 0x8E,
    end       = 0x9E,
    page_up   = 0x8F,
    page_down = 0x9F,

    up    = 0x8C,
    down  = 0x9C,
    left  = 0x9B,
    right = 0x9D,

    // Modifiers
    ctrl  = 0x88,
    alt   = 0x89,
    shift = 0x8A,
    caps  = 0x8B,

    _,
    // zig fmt: on
};

pub const lookup = blk: {
    const size = sdl.SDL_NUM_SCANCODES;
    var result: [size]?PhysicalKey = .{null} ** size;

    result[sdl.SDL_SCANCODE_BACKSPACE] = PhysicalKey.backspace;
    result[sdl.SDL_SCANCODE_TAB] = PhysicalKey.tab;
    result[sdl.SDL_SCANCODE_RETURN] = PhysicalKey.enter;
    result[sdl.SDL_SCANCODE_ESCAPE] = PhysicalKey.escape;

    for ('0'..'9' + 1) |i| {
        const scancode = @field(sdl, "SDL_SCANCODE_" ++ .{i});
        const physical_key = @field(PhysicalKey, "_" ++ .{i});
        result[scancode] = physical_key;
    }

    for ('A'..'Z' + 1) |i| {
        const scancode = @field(sdl, "SDL_SCANCODE_" ++ .{i});
        const physical_key = @field(PhysicalKey, &.{std.ascii.toLower(i)});
        result[scancode] = physical_key;
    }

    result[sdl.SDL_SCANCODE_GRAVE] = PhysicalKey.grave;
    result[sdl.SDL_SCANCODE_MINUS] = PhysicalKey.minus;
    result[sdl.SDL_SCANCODE_EQUALS] = PhysicalKey.equals;
    result[sdl.SDL_SCANCODE_LEFTBRACKET] = PhysicalKey.left_bracket;
    result[sdl.SDL_SCANCODE_RIGHTBRACKET] = PhysicalKey.right_bracket;
    result[sdl.SDL_SCANCODE_SEMICOLON] = PhysicalKey.semicolon;
    result[sdl.SDL_SCANCODE_APOSTROPHE] = PhysicalKey.apostrophe;
    result[sdl.SDL_SCANCODE_BACKSLASH] = PhysicalKey.backslash;
    result[sdl.SDL_SCANCODE_COMMA] = PhysicalKey.comma;
    result[sdl.SDL_SCANCODE_PERIOD] = PhysicalKey.period;
    result[sdl.SDL_SCANCODE_SLASH] = PhysicalKey.slash;
    result[sdl.SDL_SCANCODE_SPACE] = PhysicalKey.space;

    result[sdl.SDL_SCANCODE_INSERT] = PhysicalKey.insert;
    result[sdl.SDL_SCANCODE_DELETE] = PhysicalKey.delete;
    result[sdl.SDL_SCANCODE_HOME] = PhysicalKey.home;
    result[sdl.SDL_SCANCODE_END] = PhysicalKey.end;
    result[sdl.SDL_SCANCODE_PAGEUP] = PhysicalKey.page_up;
    result[sdl.SDL_SCANCODE_PAGEDOWN] = PhysicalKey.page_down;
    result[sdl.SDL_SCANCODE_UP] = PhysicalKey.up;
    result[sdl.SDL_SCANCODE_DOWN] = PhysicalKey.down;
    result[sdl.SDL_SCANCODE_LEFT] = PhysicalKey.left;
    result[sdl.SDL_SCANCODE_RIGHT] = PhysicalKey.right;

    result[sdl.SDL_SCANCODE_LCTRL] = PhysicalKey.ctrl;
    result[sdl.SDL_SCANCODE_RCTRL] = PhysicalKey.ctrl;
    result[sdl.SDL_SCANCODE_LALT] = PhysicalKey.alt;
    result[sdl.SDL_SCANCODE_RALT] = PhysicalKey.alt;
    result[sdl.SDL_SCANCODE_LSHIFT] = PhysicalKey.shift;
    result[sdl.SDL_SCANCODE_RSHIFT] = PhysicalKey.shift;
    result[sdl.SDL_SCANCODE_CAPSLOCK] = PhysicalKey.caps;

    break :blk result;
};
